@@ -23,7 +23,11 @@ import hashlib
 import base64
 import requests
 from tabulate import tabulate
-from io import BytesIO, StringIO
+
+if sys.version_info < (3, 0):
+    from StringIO import StringIO
+else:
+    from io import BytesIO, StringIO
 
 class Sdata_Name_Exeption(Exception): pass
 class Sdata_Uuid_Exeption(Exception): pass
@@ -49,6 +53,8 @@ class Data(object):
     SDATA_CTIME = "!sdata_ctime"
     SDATA_MTIME = "!sdata_mtime"
     SDATA_PARENT = "!sdata_parent"
+    SDATA_CLASS = "!sdata_class"
+    SDATA_PROJECT = "!sdata_project"
 
     def __init__(self, **kwargs):
         """create Data object
@@ -79,12 +85,13 @@ class Data(object):
         self.metadata = Metadata()
 
         # set default sdata attributes
-        self.metadata.add(self.SDATA_VERSION, __version__, dtype="str")
-        self.metadata.add(self.SDATA_NAME, "N.N.", dtype="str")
-        self.metadata.add(self.SDATA_UUID, uuid.uuid4(), dtype="str")
-        self.metadata.add(self.SDATA_CTIME, now_utc_str(), dtype="str")
-        self.metadata.add(self.SDATA_MTIME, now_utc_str(), dtype="str")
-        self.metadata.add(self.SDATA_PARENT, "", dtype="str")
+        self.metadata.add(self.SDATA_VERSION, __version__, dtype="str", description="sdata package version")
+        self.metadata.add(self.SDATA_NAME, "N.N.", dtype="str", description="name of the data object")
+        self.metadata.add(self.SDATA_UUID, "", dtype="str", description="Universally Unique Identifier")
+        self.metadata.add(self.SDATA_PARENT, "", dtype="str", description="uuid of the parent sdata object")
+        self.metadata.add(self.SDATA_CLASS, self.__class__.__name__, dtype="str", description="sdata class")
+        self.metadata.add(self.SDATA_CTIME, now_utc_str(), dtype="str", description="creation date")
+        self.metadata.add(self.SDATA_MTIME, now_utc_str(), dtype="str", description="modification date")
 
         metadata = kwargs.get("metadata")
         if metadata is not None:
@@ -99,18 +106,7 @@ class Data(object):
             self.auto_correct = True
         else:
             self.auto_correct = False
-
-        logger.debug("sdata: set auto_correct={}".format(self.auto_correct))
-
-        if kwargs.get("uuid") is not None:
-            try:
-                self.uuid = kwargs.get("uuid") # store given uuid str or generate a new uuid
-            except Sdata_Uuid_Exeption as exp:
-                if self.auto_correct is True:
-                    logger.warning("got invald uuid -> generate a new uuid")
-                    self.uuid = uuid.uuid4()
-                else:
-                    raise
+        # logger.debug("sdata: set auto_correct={}".format(self.auto_correct))
 
         if kwargs.get("name") is not None:
             self.name = kwargs.get("name")
@@ -122,9 +118,58 @@ class Data(object):
         self.table = kwargs.get("table", None)
         self._description = ""
         self.description = kwargs.get("description", "")
+        self.project = kwargs.get("project", "")
 
+        if (kwargs.get("uuid")=="" or kwargs.get("uuid") is not None) and not self.metadata.get(self.SDATA_UUID).value and kwargs.get("uuid")!="hash":
+            # logger.info("uuid in kwargs")
+            try:
+                self._set_uuid(kwargs.get("uuid")) # store given uuid str or generate a new uuid
+            except Sdata_Uuid_Exeption as exp:
+                if self.auto_correct is True:
+                    logger.warning("got invalid uuid -> generate a new uuid")
+                    self._set_uuid(uuid.uuid4())
+                else:
+                    raise
+        elif (kwargs.get("uuid")=="" or kwargs.get("uuid") is None) and self.metadata.get(self.SDATA_UUID).value != "":
+            # logger.info("uuid in metadata")
+            pass
+        elif kwargs.get("uuid")=="hash":
+            sha3_256 = self.gen_uuid_from_state()
+            # logger.info("gen uuid from sha3_256 {}".format(sha3_256))
+            new_uuid = uuid_from_str(sha3_256)
+            self._set_uuid(new_uuid.hex)
+        else:
+            # logger.info("uuid new")
+            self._set_uuid(uuid.uuid4())
 
+    def gen_uuid_from_state(self):
+        """generate the same uuid for the same data
 
+        :return: uuid
+        """
+        s = hashlib.sha3_256()
+        metadata = self.metadata.copy()
+        metadata.attributes.pop(self.SDATA_UUID)
+        metadata.attributes.pop(self.SDATA_MTIME)
+        metadata.attributes.pop(self.SDATA_CTIME)
+        metadatastr = metadata.to_json().encode(errors="replace")
+        s.update(metadatastr)
+        if self.table is not None:
+            tablestr = self.table.to_json().encode(errors="replace")
+            s.update(tablestr)
+        s.update(self.description.encode(errors="replace"))
+        return s.hexdigest()
+
+    def __eq__(self, other):
+        """compare Data checksum
+        
+        :param other: sdata.Data objecet
+        :return: True or False
+        """
+        if not isinstance(other, self.__class__):
+            logger.debug("you should not compare {} with {}!".format(self.__class__.__name__, other.__class__.__name__))
+            return False
+        return self.sha3_256 == other.sha3_256
 
     def update_mtime(self):
         """update modification time
@@ -287,6 +332,21 @@ class Data(object):
 
     name = property(fget=_get_name, fset=_set_name, doc="name of the object")
 
+    def _get_project(self):
+        return self.metadata.get(self.SDATA_PROJECT).value
+
+    def _set_project(self, value):
+        if isinstance(value, str):
+            try:
+                self.metadata.set_attr(self.SDATA_PROJECT, str(value)[:256])
+            except ValueError as exp:
+                logger.warning("data.project: %s" % exp)
+        else:
+            # self._name = str(value)[:256]
+            self.metadata.set_attr(self.SDATA_PROJECT, str(value)[:256])
+
+    project = property(fget=_get_project, fset=_set_project, doc="name of the project")
+
     def _get_description(self):
         return self._description
 
@@ -341,6 +401,23 @@ class Data(object):
     table = property(fget=_get_table, fset=_set_table, doc="table object(pandas.DataFrame)")
     df = table
 
+    def description_to_df(self):
+        """get description as DataFrame
+
+        :return: DataFrame of description lines
+        """
+        return pd.DataFrame(self.description.splitlines())
+
+    def description_from_df(self, df):
+        """set description from DataFrame of lines
+
+        :return:
+        """
+        if df is not None and isinstance(df, pd.DataFrame) and len(df)>0:
+            lines = df.iloc[:, 0]
+            lines = lines.astype(str)
+            self.description = "\n".join(lines.values)
+
     def to_folder(self, path, dtype="csv"):
         """export data to folder
 
@@ -488,7 +565,16 @@ class Data(object):
     @property
     def osname(self):
         """:returns: os compatible name (ascii?)"""
-        return self.name.replace(" ", "_").lower()
+        return self.asciiname.lower()
+
+    @property
+    def asciiname(self):
+        name = copy.copy(self.name)
+        mapper = [("ä", "ae"), ("ö", "oe"), ("ü", "ue"), ("Ä", "Ae"), ("Ö", "Oe"), ("Ü", "Ue"),
+                  ("ß", "sz"), (" ", "_"), ("/", "_"), ("\\", "_")]
+        for k, v in mapper:
+            name = name.replace(k, v)
+        return name.encode('ascii', 'replace').decode("ascii")
 
     def verify_attributes(self):
         """check mandatory attributes"""
@@ -876,6 +962,58 @@ class Data(object):
         data.table = df
         return data
 
+    def to_hdf5(self, filepath, **kwargs):
+        """export sdata.Data to hdf5
+
+        :param filepath:
+        :param complib: default='zlib' ['zlib', 'lzo', 'bzip2', 'blosc', 'blosc:blosclz', 'blosc:lz4', 'blosc:lz4hc', 'blosc:snappy', 'blosc:zlib', 'blosc:zstd']
+        :param complevel: default=9 [0-9]
+
+        :return:
+        """
+        if not isinstance(self.df, pd.DataFrame):
+            df = pd.DataFrame()
+        else:
+            df = self.df
+        kwargs["mode"] = "w"
+        if kwargs.get("complib") is None:
+            kwargs["complib"] = "zlib"
+
+        if kwargs.get("complevel") is None:
+            kwargs["complevel"] = 9
+
+        with pd.HDFStore(filepath, **kwargs) as hdf:
+            hdf.put('metadata'.format(self.uuid), self.metadata.df, format='fixed', data_columns=True)
+            hdf.put('table'.format(self.uuid), df, format='fixed', data_columns=True)
+            hdf.put('description'.format(self.uuid), self.description_to_df(), format='fixed', data_columns=True)
+
+
+    @classmethod
+    def from_hdf5(cls, filepath, **kwargs):
+        """import sdata.Data from hdf5
+
+        :param filepath:
+        :param sep: separator (default=";")
+        :return: sdata.Data
+        """
+        if not os.path.exists:
+            logger.error("hdf5 file '' notr available".format(filepath))
+            return
+        
+        with pd.HDFStore(filepath, mode="r+") as hdf:
+            metadata_path = "/metadata".format(uuid)
+            table_path = "/table".format(uuid)
+            description_path = "/description".format(uuid)
+            df_metadata = hdf.get(metadata_path)
+            df_table = hdf.get(table_path)
+            df_description = hdf.get(description_path)
+            metadata = Metadata.from_dataframe(df_metadata)
+            # logger.debug("hdf {}".format(metadata.get("!sdata_uuid").value))
+            data = Data(metadata=metadata, table=df_table)
+            data.description_from_df(df_description)
+
+        return data
+
     def to_html(self, filepath, xlsx=True, style=None):
         """export Data to html
 
