@@ -9,6 +9,7 @@ import json
 import os
 import hashlib
 import re
+import copy
 from sdata.contrib.sortedcontainers.sorteddict import SortedDict
 
 def extract_name_unit(value):
@@ -109,7 +110,10 @@ class Attribute(object):
             if self.dtype != dtype.__name__:
                 # logger.debug("guess dtype for ``: ``".format(value, dtype.__name__))
                 self.dtype = dtype.__name__
-            if not value and self.dtype not in ["int", "float", "bool"]:
+
+            if value == "" and self.dtype in ["str"]:
+                self._value = ""
+            elif not value and self.dtype not in ["int", "float", "bool"]:
                 self._value = None
             elif not value and self.dtype in ["int", "float"]:
                 self._value = np.nan
@@ -341,41 +345,60 @@ class Metadata(object):
             d[attr.name] = attr.to_dict()
         return d
 
+    @staticmethod
+    def guess_dtype_from_value(value):
+        """guess dtype from value,
+        e.g.
+        '1.23' -> 'float'
+        'otto1.23' -> 'str'
+        1 -> 'int'
+        False -> 'bool'
+
+        :param value:
+        :return: str ['int', 'float', 'bool', 'str']
+        """
+        if value.__class__.__name__ in ["int", "float", "bool"]:
+            return value, value.__class__.__name__
+        elif value in ["False", "True", "true", "false"]:
+            return value, 'bool'
+        try:
+            value = int(value)
+            return value, value.__class__.__name__
+        except:
+            pass
+        try:
+            value = float(value)
+            return value, value.__class__.__name__
+        except:
+            pass
+        return str(value), "str"
+
     def update_from_dict(self, d):
         """set attributes from dict"""
         for k, v in d.items():
-            if isinstance(v, (str,)):
+            value, dtype = self.guess_dtype_from_value(v)
+            if dtype in ["float", "int", "bool"]:
+                v = {"name":k, "value":value, "dtype":dtype, "unit":"", "description":"", "label":""}
+            elif isinstance(v, (str,)):
                 v = {"name":k, "value":v, "dtype":"str", "unit":"", "description":"", "label":""}
             elif hasattr(v, "keys"):
 
-                dtype = v.get("dtype", "str")
+                dtype = v.get("dtype", self.guess_dtype_from_value(v.get("value"))[1])
                 value = v.get("value")
 
                 v = {"name":k, "value":value, "dtype":dtype,
                      "unit":v.get("unit", ""), "description":v.get("description", ""),
                      "label":v.get("label", "")}
             else:
-                v = {"name":k, "value":v, "dtype":"", "unit":"", "description":"", "label":""}
+                v, dtype = self.guess_dtype_from_value(v)
+                v = {"name":k, "value":v, "dtype":dtype, "unit":"", "description":"", "label":""}
             self.set_attr(**v)
 
     @classmethod
     def from_dict(cls, d):
         """setup metadata from dict"""
         metadata = cls()
-        for k, v in d.items():
-            if isinstance(v, (str,)):
-                v = {"name":k, "value":v, "dtype":"str", "unit":"", "description":"", "label":""}
-            elif hasattr(v, "keys"):
-
-                dtype = v.get("dtype", "str")
-                value = v.get("value")
-
-                v = {"name":k, "value":value, "dtype":dtype,
-                     "unit":v.get("unit", ""), "description":v.get("description", ""),
-                     "label":v.get("label", "")}
-            else:
-                v = {"name":k, "value":v, "dtype":"", "unit":"", "description":"", "label":""}
-            metadata.set_attr(**v)
+        metadata.update_from_dict(d)
         return metadata
 
     def to_dataframe(self):
@@ -552,6 +575,11 @@ class Metadata(object):
         """
         return list(self._attributes.items())
 
+
+    def copy(self):
+        """returns a deep copy"""
+        return copy.deepcopy(self)
+
     @property
     def size(self):
         """return number uf Attribute"""
